#import "LogRocketNativeModule.h"
#import <React/RCTLog.h>
#import <LogRocket/LogRocket-Swift.h>

@implementation LogRocketNativeModule

RCT_EXPORT_MODULE();

static NSMutableArray *fullMessageArray = nil;

- (id)init {
  self = [super init];

  if (self) {
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleError:) name:@"LogRocketSDKError" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleSessionAccepted:) name:@"LogRocketSDKSessionAccepted" object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(handleStartSessionFromInactive:) name:@"LogRocketSDKStartSessionFromInactive" object:nil];
  }

  return self;
}

- (void)handleError:(NSNotification*)notification {
  NSString *errorMessage = notification.userInfo[@"errorMessage"];
  NSNumber *shouldDisableAndFreeMemory = notification.userInfo[@"shouldDisableAndFreeMemory"];
  NSNumber *shouldWarnAboutReduxSize = notification.userInfo[@"shouldWarnAboutReduxSize"];

  [self sendEventWithName:@"LogRocketSDKOnError" body:@{@"errorMessage": errorMessage, @"shouldDisableAndFreeMemory": shouldDisableAndFreeMemory, @"shouldWarnAboutReduxSize": shouldWarnAboutReduxSize}];
}

- (void)handleSessionAccepted:(NSNotification*)notification {
  NSString *sessionURL = notification.userInfo[@"sessionURL"];

  [self sendEventWithName:@"LogRocketSDKOnSessionAccepted" body:@{@"sessionURL": sessionURL}];
}

- (void)handleStartSessionFromInactive:(NSNotification*)notification {
  [self sendEventWithName:@"LogRocketSDKOnStartSessionFromInactive" body:@{}];
}

+ (BOOL)requiresMainQueueSetup {
  return YES;
}

- (NSArray<NSString *> *)supportedEvents {
  return @[@"LogRocketSDKOnError", @"LogRocketSDKOnSessionAccepted", @"LogRocketSDKOnStartSessionFromInactive", @"LogRocketSDKOnDisableReduxCapture"];
}

+ (nullable LROConfiguration *)
                  createConfig:(NSString *)appID
                  config:(NSDictionary *)config
{
    NSString *serverURL = config[@"serverURL"];
    NSString *updateId = config[@"updateId"];
    NSString *expoChannel = config[@"expoChannel"];
    NSMutableSet<NSString *> *allowTags = [NSMutableSet setWithArray:config[@"allowTags"]];
    [allowTags addObject:@"lr-show"];
    NSMutableSet<NSString *> *redactionTags = [NSMutableSet setWithArray:config[@"redactionTags"]];
    [redactionTags addObject:@"lr-hide"];

    LROConfiguration *configuration = [[LROConfiguration alloc] initWithAppID:appID];

    configuration.serverURL = serverURL;
    configuration.updateId = updateId;
    configuration.expoChannel = expoChannel;
    configuration.allowTags = allowTags;
    configuration.redactionTags = redactionTags;
    configuration.isReactNative = YES;
    configuration.networkCaptureEnabled = NO;
    configuration.requestSanitizer = nil;
    configuration.responseSanitizer = nil;

    #ifdef RCT_NEW_ARCH_ENABLED
      configuration.isRNNewArchEnabled = YES;
    #endif

    // Entries in the dictionary can be NSNull. Especially, on React-Native, this can happen.
    id enablePersistence = config[@"enablePersistence"];
    if (enablePersistence != nil && ![enablePersistence isEqual:[NSNull null]]) {
      configuration.persistenceEnabled = [enablePersistence boolValue];
    } else {
      configuration.persistenceEnabled = YES;
    }

    if ([config[@"textSanitizer"] isEqualToString:@"excluded"]) {
      configuration.textSanitizer = LROSanitizerTypeExcluded;
    } else {
      configuration.textSanitizer = LROSanitizerTypeNone;
    }

    id enableIPCapture = config[@"enableIPCapture"];
    if (enableIPCapture != nil && ![enableIPCapture isEqual:[NSNull null]]) {
      configuration.ipCaptureEnabled = [enableIPCapture boolValue];
    } else {
      configuration.ipCaptureEnabled = YES;
    }

    id experimentalBidiCapture = config[@"experimentalBidiCapture"];
    if (experimentalBidiCapture != nil && ![experimentalBidiCapture isEqual:[NSNull null]]) {
      configuration.experimentalBidiCapture = [experimentalBidiCapture boolValue];
    } else {
      configuration.experimentalBidiCapture = NO;
    }

    id iosThroughputTrackingEnabled = config[@"iosThroughputTrackingEnabled"];
    if (iosThroughputTrackingEnabled != nil && ![iosThroughputTrackingEnabled isEqual:[NSNull null]]) {
      configuration.throughputTrackingEnabled = [iosThroughputTrackingEnabled boolValue];
    } else {
      configuration.throughputTrackingEnabled = YES;
    }

    id iosExtendedViewTextCapture = config[@"iosExtendedViewTextCapture"];
    if (iosExtendedViewTextCapture != nil && ![iosExtendedViewTextCapture isEqual:[NSNull null]]) {
      configuration.extendedViewTextCaptureEnabled = [iosExtendedViewTextCapture boolValue];
    } else {
      configuration.extendedViewTextCaptureEnabled = YES;
    }

    id captureRedactedViewTouches = config[@"captureRedactedViewTouches"];
    if (captureRedactedViewTouches != nil && ![captureRedactedViewTouches isEqual:[NSNull null]]) {
      configuration.captureRedactedViewTouches = [captureRedactedViewTouches boolValue];
    } else {
      configuration.captureRedactedViewTouches = YES;
    }

    id forceCleanStart = config[@"forceCleanStart"];
    if (forceCleanStart != nil && ![forceCleanStart isEqual:[NSNull null]]) {
      configuration.forceCleanStart = [forceCleanStart boolValue];
    } else {
      configuration.forceCleanStart = NO;
    }

    id disableScrollDelegate = config[@"disableScrollDelegate"];
    if (disableScrollDelegate != nil && ![disableScrollDelegate isEqual:[NSNull null]]) {
      configuration.disableScrollDelegate = [disableScrollDelegate boolValue];
    } else {
      configuration.disableScrollDelegate = NO;
    }

    id registerTouchHandlers = config[@"registerTouchHandlers"];
    if (registerTouchHandlers != nil && ![registerTouchHandlers isEqual:[NSNull null]]) {
      configuration.registerTouchHandlers = [registerTouchHandlers boolValue];
    } else {
      configuration.registerTouchHandlers = YES;
    }

    id viewScanningEnabled = config[@"viewScanningEnabled"];
    if (viewScanningEnabled != nil && ![viewScanningEnabled isEqual:[NSNull null]]) {
      configuration.viewScanningEnabled = [viewScanningEnabled boolValue];
    } else {
      configuration.viewScanningEnabled = YES;
    }

    id enableAutomaticLifecycleCapture = config[@"enableAutomaticLifecycleCapture"];
    if (enableAutomaticLifecycleCapture != nil && ![enableAutomaticLifecycleCapture isEqual:[NSNull null]]) {
      configuration.enableAutomaticLifecycleCapture = [enableAutomaticLifecycleCapture boolValue];
    } else {
      configuration.enableAutomaticLifecycleCapture = YES;
    }

    id nativeLogCaptureEnabled = config[@"nativeLogCaptureEnabled"];
    if (nativeLogCaptureEnabled != nil && ![nativeLogCaptureEnabled isEqual:[NSNull null]]) {
      configuration.logCaptureEnabled = [nativeLogCaptureEnabled boolValue];
    } else {
      configuration.logCaptureEnabled = YES;
    }

    id proxyConfiguration = config[@"iosProxyConfiguration"];
    if (
        proxyConfiguration != nil &&
        ![proxyConfiguration isEqual:[NSNull null]] &&
        [proxyConfiguration isKindOfClass: [NSDictionary class]]
        ) {
      LROProxyConfigurationBuilder *builder = [[LROProxyConfigurationBuilder alloc] init];

      if (proxyConfiguration[@"proxyUsername"]) {
        [builder proxyUsername:proxyConfiguration[@"proxyUsername"]];
      }

      if (proxyConfiguration[@"proxyPassword"]) {
        [builder proxyPassword:proxyConfiguration[@"proxyPassword"]];
      }

      if (proxyConfiguration[@"httpEnable"]) {
        [builder httpEnable:proxyConfiguration[@"httpEnable"]];
      }

      if (proxyConfiguration[@"httpProxy"]) {
        [builder httpProxy:proxyConfiguration[@"httpProxy"]];
      }

      if (proxyConfiguration[@"httpPort"]) {
        [builder httpPort: [proxyConfiguration[@"httpPort"] unsignedIntValue]];
      }

      configuration.proxyConfiguration = builder;
    }

    NSString *logLevel = config[@"logLevel"];
    if (logLevel != nil) {
      configuration.logLevel = [LROSDK logLevelFromString:logLevel];
    }

    if ([config[@"uploadIntervalMs"] isKindOfClass:[NSNumber class]]) {
      configuration.uploadIntervalMs = [config[@"uploadIntervalMs"] unsignedIntValue];
    }

    if ([config[@"viewCaptureTimeoutThreshold"] isKindOfClass:[NSNumber class]]) {
      configuration.viewCaptureTimeoutThreshold = [config[@"viewCaptureTimeoutThreshold"] unsignedIntValue];
    }

    if ([config[@"viewCaptureAdditionalWireframeDuration"] isKindOfClass:[NSNumber class]]) {
      configuration.viewCaptureTimeoutThreshold = [config[@"viewCaptureAdditionalWireframeDuration"] unsignedIntValue];
    }
    return configuration;
}

RCT_EXPORT_METHOD(initWithConfig:(NSString *)appID
                  config:(NSDictionary *)config
                  resolver:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
  @try {
    LROConfiguration *configuration = [LogRocketNativeModule createConfig:appID config:config];

    BOOL result = [LROSDK initializeInternalWithConfiguration:configuration];

    resolve(@(result));
  }

  @catch (NSException *e) {
    NSLog(@"Failed to start LogRocket SDK");
    resolve(@(false));
  }
}

RCT_EXPORT_METHOD(identifyWithTraits:(NSString *)userID
                  userInfo:(NSDictionary *)userInfo
                  isAnonymous:(nonnull NSNumber *)isAnonymous)
{
    @try {
        if ([isAnonymous intValue] >= 1) {
          [LROSDK identifyAsAnonymousWithUserID:userID userInfo:userInfo];
        } else {
          [LROSDK identifyWithUserID:userID userInfo:userInfo];
        }
    }
    @catch (NSException *e) {
        NSLog(@"LogRocket: Failed to identify user.");
    }
}

RCT_EXPORT_METHOD(shutdown:(RCTPromiseResolveBlock)resolve
                  rejecter:(RCTPromiseRejectBlock)reject)
{
  @try {
    [LROSDK endSession];
    resolve(@(true));
  }

  @catch (NSException *e) {
    NSLog(@"Failed to shutdown LogRocket SDK");
    resolve(@(false));
  }
}

RCT_EXPORT_METHOD(track:(NSString *)customEventName eventProperties:(NSDictionary *)eventProperties)
{
  @try {
    LROCustomEventBuilder *builder = [[LROCustomEventBuilder alloc] init:customEventName];

    for (NSString *key in eventProperties) {
      NSDictionary *value = [eventProperties objectForKey:key];

      if ([value objectForKey:@"doubleVal"]) {
        NSArray *doubleArray = [value objectForKey:@"doubleVal"];
        [builder putDoubleArray:key value:doubleArray];
      } else if ([value objectForKey:@"boolVal"]) {
        NSArray *boolArray = [value objectForKey:@"boolVal"];
        [builder putBoolArray:key value:boolArray];
      } else if ([value objectForKey:@"stringVal"]) {
        NSArray *stringArray = [value objectForKey:@"stringVal"];
        [builder putStringArray:key value:stringArray];
      }
    }
    [LROSDK track:builder];
  }
  @catch (NSException *e) {
    NSLog(@"Failed to track custom event");
  }
}

RCT_EXPORT_METHOD(pauseViewCapture)
{
  @try {
    [LROSDK pauseViewCapture];
  }

  @catch (NSException *e) {
    NSLog(@"Failed to pause view capture");
  }
}

RCT_EXPORT_METHOD(unpauseViewCapture)
{
  @try {
    [LROSDK unpauseViewCapture];
  }

  @catch (NSException *e) {
    NSLog(@"Failed to unpause view capture");
  }
}

RCT_EXPORT_METHOD(tagPage:(NSString *)relativePage)
{
  @try {
    [LROSDK tagPage:relativePage];
  }
  @catch (NSException *e) {
    NSLog(@"Failed to tag page");
  }
}

RCT_EXPORT_METHOD(postMessage:(NSArray *) message)
{
  if (!fullMessageArray) {
    fullMessageArray = [NSMutableArray arrayWithArray:message];
  } else {
    [fullMessageArray addObjectsFromArray: message];
  }
}

RCT_EXPORT_METHOD(donePostingMessage)
{
  if (fullMessageArray) {
    [LROSDK rebuildMessage:fullMessageArray];
    // Set array to nil so that it get replaced with a new array when another event
    // is captured. This array instance could still be getting processed by the
    // native SDK when another event comes in.
    fullMessageArray = nil;
  } else {
    NSLog(@"Received donePostingMessage without posting any message");
  }
}

@end
